// IO.js


// open file for editing
function openFile(path) {
    if (!path) {
        path = inputFile.value;
    }
    path = trim( path );

    if (path.search(/about:mozedit/) != -1) {
        togglePreview(true);
        return;
    }

    if (path.search(/file:/) != -1) {
        path = path.replace(/file:\/\//,"");
    }

    if (path != "") {
        try {
            var f = null;

            f = new File(path);

            if (f.exists()) {
                doBackup(path);
                f.open();
                var fileData = f.read();
                f.close();

                buffer.saveBuffer(content.value, inputFile.value);
                if (isOpenNewDocumentEnabled == "true") {
                    addMenuBufferItem();
                    addTab(path);
                }
                buffer = bufferList.getBuffer(BufferList.currentBuffer);
                buffer.saveBuffer(fileData, path);
                content.value = fileData;
                inputFile.value = path;
                setModifiedState( false )
                setWindowTitle(path);
                addToHistory(path);
                buildBufferMenu(path);
                refreshPreview();
                refreshSidebar();
                doThrobber();
                updateStatus("File opened...");
            } else {
                alert("The file does not exist.");
                return false;
            }
        } catch(e) {
            dump(e +" error in openFile\n");
            return false;
        }
    } else {
        alert("Please enter a valid file name to open.");
		return false;
    }
	return true;
}



// insert file
function insertFile(path) {
    this.path = trim( path );

    if (path.search(/file:/) != -1) {
        path = path.replace(/file:\/\//,"");
    }

    if (path != "") {
        try {
            var f = null;

            f = new File(path);

            if (f.exists()) {
                doBackup(path);
                f.open();
                var fileData = f.read();
                f.close();
                return fileData;
            } else {
                alert("The file does not exist.");
                return false;
            }
        } catch(e) {
            dump(e +"error in openFile");
            return false;
        }
    } else {
        alert("Please enter a valid file name to open.");
        return false;
    }
    return true;
}


function openFilePickerLogic() {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].
        createInstance( nsIFilePicker );
    fp.init( window, "Open File", nsIFilePicker.modeOpen );
	fp.appendFilter("Source Files","*.html; *.htm; *.txt; *.xml; *.xul; *.rdf; *.css; *.dtd; *.php; *.js; *.jsp; *.asp; *.xhtml; *.shtml; *.java; *.cpp; *.pl");
	fp.appendFilters( nsIFilePicker.filterText | nsIFilePicker.filterAll);
    var res=fp.show();
    if (res==nsIFilePicker.returnOK){
        var thefile=fp.file;
        return thefile.path;
    }
    return false;
}

function browseFilePicker() {
    var path = openFilePickerLogic();
    return path;
}


// open file using filepicker
function openFilePicker() {
    var path = openFilePickerLogic();

    if (!path) {
        return;
    }
    openFile(path);
    updateStatus("Opening file with picker..");
}


function insertFilePicker() {
    var path = openFilePickerLogic();

    if (!path) {
        return;
    }
    var textToInsert = insertFile(path);
    pasteText(textToInsert);
    updateStatus("Inserting file with picker..");
}



function save(path) {
    if (path == null) {
        path = inputFile.value;
    }
    if (path == "") {
        saveFilePicker();
        return;
    }
    var result = saveLogic(path);
    if (result) {
        inputFile.value = path;
        buffer.saveBuffer(content.value,path);
        buildBufferMenu();
        setModifiedState( false );
        addToHistory(path);
        doBackup(path);
        setWindowTitle(path);
        setTabTitle(path);
        doThrobber();
    } else {
        dump("Save failed to write in save():\n");
    }
}

// save file logic
function saveLogic(path,string) {
    var contentFile = content.value;
    if (content.value == null || content.value == "") {
        contentFile = " ";
    }
    if (string != null) {
        contentFile = string;
        if (string == "") {
            contentFile = " ";
        }
    }

    if (path != null && path != "") {
        var path = trim(path);
        var f = new File(path);

        if (f.exists() && f.isFile()) {
            f.open('w');
			try {
				f.write(contentFile);
			} catch (e) {
			    dump("empty file created.");
				f.close();
				alert("An error occured and the file was not saved.");
				return false;
			}
	        f.close();
            updateStatus("File saved...");
        } else {
            try {
                f.create();
            } catch (e) {
                dump("file not created in function save()");
                alert("The file name or path is not valid.\nThe file was not saved.");
                return false;
            }

            if (f.exists() && f.isFile()) {
                f.open('w');
                f.write(contentFile);
                updateStatus("File saved...");
                f.close();
            } else {
                alert("The file could not be saved.");
                return false;
            }
            f.close();
        }
    } else {
        alert("Please enter a file name to save.");
        return false;
    }
    return true;
}

// save file using filepicker
function saveFilePicker(hold) {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].
        createInstance( nsIFilePicker );
    fp.init( window, "Save File Dialog", nsIFilePicker.modeSave );
    fp.appendFilter("Source Files","*.html; *.htm; *.txt; *.xml; *.xul; *.rdf; *.css; *.dtd; *.php; *.js; *.jsp; *.asp; *.xhtml; *.shtml; *.java; *.cpp; *.pl");
	fp.appendFilters( nsIFilePicker.filterText | nsIFilePicker.filterAll);
    var res=fp.show();
    if (res != nsIFilePicker.returnCancel){
        var thefile=fp.file;
	    updateStatus("Saving file..");
	    path = thefile.path;
	    save(path);
        return true;
    } else if (hold != "") { //cancels close window event
        return false;
    }
    return false;
}

// Backup file on open
function doBackup(path) {
    if ( isBackupEnabled == "true" ) {
	    var u,f = null;
        u = new FileUtils();
		f = new File(path);

		try {
		    if ( f.exists() ) {
			    u.remove(path+"~");
			}
		    u.copy(path,path+"~");
		} catch(e) {
		    dump("File was not copied or deleted in doBackup()\n");
         	alert("An error occured and Auto backup could not backup your file.");
			return false;
		}
	}
	return true;
}



// remove backup files in current directory
function killBackups() {
    if ( inputFile.value == "" ) {
        alert("You need to open at least one file first before \nremoving backup files to set the directory.");
        return false;
    }
    if ( !confirm("Clicking okay will remove all Mozedit backup files in your current directory.\n"+
        "Mozedit backup files have the suffix ~. Do you want to continue?") ) {
        return false;
    }
    var f,fu,d = null;
    var fu = new FileUtils();
    var path = fu.parent(inputFile.value);
    var d = new Dir(path);

    try {
        var fileArray = d.readDir();

        if ( fileArray.length > 0 ) {
            for (x=0; x < fileArray.length; x++) {
                if ( fileArray[x].path.search(/~$/) == -1 ) {
                    continue;
                }
                try {
                    dump( "Removing backup files in killBackups(): \n"+fileArray[x].path + "\n");
                    var f = new File(fileArray[x].path);
                    if ( f.isFile) {
                        fu.remove(fileArray[x].path);
                    }
                    doThrobber();
                } catch(e) {
                    dump(e+" Files could not be removed in killBackups()\n");
                    return false;
                }
            }
        }
        fileArray = null;
        return true;
    } catch(e) {
        dump(e+" error in killBackups()");
    }
}
